@@ -1,4 +1,9 @@
+#!/usr/bin/python3
 """Call of Cthulhu character generator
+
+Example:
+
+>>> Character()
 """
 import random
 import os
@@ -10,58 +15,23 @@ from .utils import OCCUPATIONS_DATA
 from .utils import BASIC_SKILLS
 from .utils import ALL_SKILLS
 from .utils import CATEGORY_SKILLS
+from .utils import TRANSLATION_DICT
+from .utils import CATEGORY_SKILLS_LIST
+from .utils import AGE_RANGE
+from .utils import YEAR_RANGE
+# from .utils import *
 
 _THIS_FOLDER = os.path.dirname(os.path.abspath(__file__))
 POP_PIRAMID_PATH = os.path.abspath(os.path.join(_THIS_FOLDER, 'data', 'popPiramid.json'))
 
 
-TRANSLATION_DICT = {
-    "a": "art/craft",
-    "s": "science",
-    "f": "fighting",
-    "g": "firearms",
-    "i": "interpersonal",
-    "l": "language",
-    "*": None,
-    "p": "professor"
-    # just for professor occ. (i ingored doctor, and give him 2*)
-}
-
-CATEGORY_SKILLS_LIST = [
-    "language", "art/craft", "science", "fighting", "firearms"
-]
-
-AGE_RANGE = (
-    (15, 19),
-    (20, 24),
-    (25, 29),
-    (30, 34),
-    (35, 39),
-    (40, 44),
-    (45, 49),
-    (50, 54),
-    (55, 59),
-    (60, 64),
-    (65, 69),
-    (70, 74),
-    (75, 79),
-    (80, 84),
-    (85, 89),
-    (90, 94),
-    (95, 99),
-)
-YEAR_RANGE = (1950, 1955, 1960, 1965, 1970, 1975, 1980, 1985, 1990, 1995, 2000,
-              2005, 2010, 2015, 2020)
-
-
 class Character():
-    _MIN_AGE = 15
-    _MAX_AGE = 90
-    __SEX_OPTIONS = ['M', 'F', None]
-    OCCUPATIONS_GROUPS = OCCUPATIONS_GROUPS.copy()
-    OCCUPATIONS_LIST = OCCUPATIONS_LIST.copy()
-    max_skill_level = 90
-
+    _MIN_AGE: int = 15
+    _MAX_AGE: int = 90
+    __SEX_OPTIONS: list = ['M', 'F', None]
+    OCCUPATIONS_GROUPS: list = OCCUPATIONS_GROUPS.copy()
+    OCCUPATIONS_LIST: list = OCCUPATIONS_LIST.copy()
+    max_skill_level: int = 90
 
     def __init__(
             self,
@@ -86,6 +56,7 @@ class Character():
         self._first_name = randname.first_name(self._year, self._sex, self._country, self._weights) if not first_name else first_name
         self._last_name = randname.first_name(self._year, self._sex, self._country, self._weights) if not last_name else last_name
         # characteristics
+        self._characteristics = {}
         self._str = 0
         self._con = 0
         self._siz = 0
@@ -95,7 +66,7 @@ class Character():
         self._int = 0
         self._pow = 0
         self._move_rate = 0
-        self._set_characteristics()
+        self.set_characteristics()
         # derived atributes
         self._sanity_points = 0
         self._magic_points = 0
@@ -121,7 +92,7 @@ class Character():
             self._edu * 2 + self._str * 2,  # 5
         )
 
-        self._occupation = self.get_occupation(self._occupation)
+        self._occupation = self.set_occupation(self._occupation)
         self._occupation_points = self.get_skill_points(self._occupation)
         self._hobby_points = self._int * 2 # Create public function for that
 
@@ -131,22 +102,20 @@ class Character():
         self.hobby_skills_list = []
 
         ALL_SKILLS.update({"doge": self._dex // 2, "language (own)": self._edu})
-        self.skills_dict = self.get_skills_dict()
+        self._skills = self.set_skills_dict()
 
         # credit rating
         # assigning points to credit rating
         credit_rating_points = random.randint(
             *OCCUPATIONS_DATA[self._occupation]['credit_rating'].copy())
         self._occupation_points -= credit_rating_points
-        self.skills_dict.setdefault('credit rating', credit_rating_points)
+        self._skills.setdefault('credit rating', credit_rating_points)
 
         # combat values
         self._damage_bonus = ""
         self._build = 0
         self._doge = 0
-        self.combat_values()
-
-
+        self.set_combat_values()
 
     @property
     def year(self):
@@ -172,6 +141,75 @@ class Character():
     def occupation(self):
         return self._occupation
 
+    @property
+    def characteristics(self):
+        return self._characteristics
+
+    @property
+    def strength(self):
+        return self._str
+
+    @property
+    def condition(self):
+        return self._con
+
+    @property
+    def size(self):
+        return self._siz
+
+    @property
+    def dexterity(self):
+        return self._dex
+
+    @property
+    def apperance(self):
+        return self._app
+
+    @property
+    def edducation(self):
+        return self._edu
+
+    @property
+    def intelligence(self):
+        return self._int
+
+    @property
+    def power(self):
+        return self._pow
+
+    @property
+    def move_rate(self):
+        return self._move_rate
+
+    @strength.setter
+    def strength(self, new_strength):
+        try:
+            int(new_strength)
+        except ValueError:
+            raise ValueError("Invalid strength. Strength must me an integer")
+        if new_strength < 0:
+            raise ValueError("Strength cannot be less than 0")
+        self._str = new_strength
+
+    @condition.setter
+    def condition(self, new_condition):
+        try:
+            int(new_condition)
+        except ValueError:
+            raise ValueError("Invalid condition. Strength must me an integer")
+        if new_condition < 0:
+            raise ValueError("Strength cannot be less than 0")
+        self._con = new_condition
+
+
+    @year.setter
+    def year(self, new_year):
+        try:
+            int(new_year)
+            self._year = new_year
+        except ValueError:
+            raise ValueError("invalid year. year must be integer")
+
     @sex.setter
     def sex(self, new_sex):
         if new_sex in ['M', 'F', None]:
@@ -186,6 +224,18 @@ class Character():
         else:
             raise ValueError(f"age not in range: [{self._MIN_AGE}, {self._MAX_AGE}]")
 
+    @country.setter
+    def country(self, new_country):
+        # TO DO
+        self._country = new_country
+
+    @weights.setter
+    def weights(self, new_weights):
+        if isinstance(new_weights, bool):
+            self._weights = new_weights
+        else:
+            raise ValueError("weights must be a bool")
+
     @occupation.setter
     def occupation(self, new_occupation):
         # TO DO
@@ -193,19 +243,19 @@ class Character():
     
     @staticmethod
     def set_sex(sex):
-        """Set character Sex
-
-        :param sex: new sex
-        :type sex: [type]
-        :return: [description]
-        :rtype: [type]
-        """
         if sex is None:
             return random.choice(['M', 'F'])
         else:
             return sex.upper()
 
     def set_age(self, age=None):
+        """Set age  
+
+        :param age: new age, defaults to None
+        :type age: int, optional
+        :return: new age
+        :rtype: int
+        """
         if age is None:
             variable_year = self._year
             if variable_year < 1950:
@@ -221,15 +271,20 @@ class Character():
                     self._sex][3:-1]
                 age_range = random.choices(age_population,
                                            weights=age_weights)[0]
-                self.age = random.randint(*age_range)
-            return self.age
+                self._age = random.randint(*age_range)
+            return self._age
         else:
-            self.age = age
-            return self.age
+            self._age = age
+            return self._age
 
     ###################### CHARACTERISTICS ########################
 
-    def _set_characteristics(self):
+    def set_characteristics(self):
+        """[summary]
+
+        :return: [description]
+        :rtype: [type]
+        """
         # Main characteristics
         self._str = random.randint(15, 90)
         self._con = random.randint(15, 90)
@@ -263,6 +318,20 @@ class Character():
         self._characteristic_test('_edu', mod_edu)
         self._correct_move_rate(mod_move_rate)
 
+        self._characteristics = {
+            "str": self._str,
+            "con": self._con,
+            "siz": self._siz,
+            "dex": self._dex,
+            "app": self._app,
+            "edu": self._edu,
+            "int": self._int,
+            "pow": self._pow,
+            "move_rate": self._move_rate,
+        }
+
+        return self._characteristics
+
     def _set_derived_attributes(self):
         # Warning! First use add_characteristics method
         self._sanity_points = self._pow
@@ -313,7 +382,14 @@ class Character():
 
     ########################## OCCUPATION ###########################
 
-    def get_occupation(self, occupation):
+    def set_occupation(self, occupation):
+        """[summary]
+
+        :param occupation: [description]
+        :type occupation: [type]
+        :return: [description]
+        :rtype: [type]
+        """
         if occupation == 'random':
             self._occupation = random.choice(self.occupations_list)
             return self._occupation
@@ -332,6 +408,13 @@ class Character():
     ########################### SKILLS #####################3########
 
     def get_skill_points(self, occupation: str):
+        """[summary]
+
+        :param occupation: [description]
+        :type occupation: str
+        :return: [description]
+        :rtype: [type]
+        """
         group_index = [
             index for index, group in enumerate(self.OCCUPATIONS_GROUPS) if occupation in group
         ]
@@ -339,25 +422,31 @@ class Character():
 
         return max(points)
 
-    def get_skills_dict(self):
-        self.skills_dict = {}
+    def set_skills_dict(self):
+        """[summary]
+
+        :return: [description]
+        :rtype: [type]
+        """
+        self._skills = {}
         occupation_input_list = OCCUPATIONS_DATA[
             self._occupation]["skills"].copy()
-        self.occupation_skills_list = self.get_skills_list(
+        self.occupation_skills_list = self._get_skills_list(
             occupation_input_list)
 
         # categories = [f"1{cat}" for cat in TRANSLATION_DICT.keys()]
         # hobby_input_list = list(BASIC_SKILLS.keys()) + categories
         hobby_input_list = list(BASIC_SKILLS.keys())
-        self.hobby_skills_list = self.get_skills_list(hobby_input_list)
+        self.hobby_skills_list = self._get_skills_list(hobby_input_list)
 
         self._assign_skill_points(self._occupation_points,
                                   self.occupation_skills_list)
         self._assign_skill_points(self._hobby_points, self.hobby_skills_list)
-        self.skills_dict = self.filter_skills(self.skills_dict)
-        return self.skills_dict
+        self._skills = self.filter_skills(self._skills)
+        
+        return self._skills
 
-    def get_skills_list(self, input_list: list):
+    def _get_skills_list(self, input_list: list):
         skills_list = []
         skills_list += list(
             filter(
@@ -399,40 +488,40 @@ class Character():
     def _assign_skill_points(self, points: int, skills_list: list):
         for skill in skills_list:
             if skill in ALL_SKILLS:
-                self.skills_dict.setdefault(skill, ALL_SKILLS[skill])
+                self._skills.setdefault(skill, ALL_SKILLS[skill])
             else:
-                self.skills_dict.setdefault(skill, 1)
+                self._skills.setdefault(skill, 1)
 
         while points:
             skill = random.choice(skills_list)
-            if points <= self.max_skill_level - self.skills_dict[skill]:
+            if points <= self.max_skill_level - self._skills[skill]:
                 points_allocation = random.randint(0, points)
-            elif sum(list(self.skills_dict.values())) % 90 == 0:
+            elif sum(list(self._skills.values())) % 90 == 0:
                 break
-            elif self.skills_dict[skill] >= self.max_skill_level:
+            elif self._skills[skill] >= self.max_skill_level:
                 continue
             else:
                 points_allocation = random.randint(
-                    0, self.max_skill_level - self.skills_dict[skill])
-            self.skills_dict[skill] += points_allocation
+                    0, self.max_skill_level - self._skills[skill])
+            self._skills[skill] += points_allocation
             points -= points_allocation
 
-    def filter_skills(self, skills_dict: dict):
+    def filter_skills(self, _skills: dict):
         """Filter out all skills with basic value form given dict.
 
-        >>>example_dict = {'psychoanalysis': 1, 'language (spanish)': 66}
-        >>>filter_skills(example_dict)
+        >>> example_dict = {'psychoanalysis': 1, 'language (spanish)': 66}
+        >>> filter_skills(example_dict)
         {'language (spanish)': 66}
         """
         return dict(
             filter(
                 lambda item: ALL_SKILLS.get(item[
                     0], ALL_SKILLS.setdefault(item[0], 1)) != item[1],
-                skills_dict.items()))
+                _skills.items()))
 
     ######################## COMBAT VALUES #############################
 
-    def combat_values(self):
+    def set_combat_values(self):
         VALUE_MATRIX = {
             'combat_range': [64, 84, 124, 164, 204, 283, 364, 444, 524],
             'damage_bonus':
@@ -473,20 +562,20 @@ class Character():
             "sanity": {self._sanity_points},
             "luck": {self._luck},
             "magic_points": {self._magic_points},
-            "skills": {self.skills_dict},
             "combat":
                 "damage bonus": {self._damage_bonus},
                 "build": {self._build},
-                "doge": {self._doge}
+                "doge": {self._doge},
+            "skills": {self._skills},
         """
 
     def __eq__(self, o: object) -> bool:
-        pass
+        return True if self.__dict__ == o.__dict__ else False
 
-    def __repr__(self) -> str:
-        pass
+    # def __repr__(self) -> str:
+    #     pass
 
 if __name__ == "__main__":
     print(Character(first_name="Adam"))
     print(Character(age=15))
-    print(Character())
\ No newline at end of file
+    print(Character().__dict__)
\ No newline at end of file
